from werkzeug.exceptions import BadRequest, Conflict, NotFound
from sqlalchemy.exc import IntegrityError
from typing import Union, List

from flaschengeist import logger
from flaschengeist.database import db

from .models import Price, DrinkType


def update_model(obj: object, data: dict):
    for attr in data.keys():
        if hasattr(obj, attr):
            setattr(obj, attr, data[attr])


def get_drink_type(name: Union[str, int]) -> DrinkType:
    logger.debug("GET DrinkType with name {{name}}")
    drink_type = None
    if isinstance(name, int):
        drink_type = db.session.get(DrinkType, name)
    if isinstance(name, str):
        drink_type = db.session.execute(db.select(DrinkType).where(DrinkType.name == name)).scalars().one_or_none()
    if drink_type == None:
        raise BadRequest
    logger.debug("Send DrinkType with name {{name}}")
    return drink_type


def get_drink_types() -> List[DrinkType]:
    return db.session.execute(db.select(DrinkType)).scalars().all()


def set_drink_type(name: None, data: dict) -> DrinkType:
    if not "name" in data:
        raise BadRequest
    if "id" in data:
        data.pop("id")
    if name:
        drink_type = get_drink_type(name)
        update_model(drink_type, data)
    else:
        drink_type = DrinkType(**data)
        db.session.add(drink_type)
    try:
        db.session.commit()
    except IntegrityError:
        logger.warn("Drink type with data {{data}} exeists.")
        db.session.rollback()
        raise Conflict
    return drink_type


def delete_drink_type(id: int):
    drink_type = get_drink_type(id)
    try:
        db.session.delete(drink_type)
        db.session.commit()
    except IntegrityError:
        logger.error("Can not delete drink_type with id {{id}}")
        db.session.rollback()
        raise Conflict


def get_price(name: Union[str, int]) -> Price:
    logger.debug("GET Price with name {{name}}")
    price = None
    if isinstance(name, int):
        price = db.session.get(Price, name)
    if isinstance(name, str):
        price = db.session.execute(db.select(Price).where(Price.name == name)).scalars().one_or_none()
    if price == None:
        raise BadRequest
    logger.debug("Send Price with name {{name}}")
    return price


def get_prices() -> List[Price]:
    return db.session.execute(db.select(Price)).scalars().all()


def set_price(name: None, data: dict):
    if not any(x in data for x in ["name", "price", "drink_type_id"]):
        raise BadRequest
    if "id" in data:
        data.pop("id")
    if "drink_type" in data:
        data.pop("drink_type")
    data["_drink_type_id"] = data["drink_type_id"]
    if name:
        price = get_price(name)
        update_model(price, data)
    else:
        price = Price()
        update_model(price, data)
        db.session.add(price)
    try:
        db.session.commit()
    except IntegrityError:
        logger.error("Price with data {{date}} exists.")
        db.session.rollback()
        raise Conflict
    return price


def delete_price(id: int):
    price = get_price(id)
    try:
        db.session.delete(price)
        db.session.commit()
    except IntegrityError:
        logger.error("Can not delete drink_price with id {{id}}")
        db.session.rollback()
        raise Conflict
